from rest_framework.generics import ListAPIView, RetrieveUpdateDestroyAPIView, ListAPIView
from rest_framework import permissions

from django.http import HttpResponse, JsonResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework.parsers import JSONParser
from reqman.apps.reqtool.rest_api.serializers.requirements import BoilerplateGroupClassesInstancesSerializer, BoilerplateGroupDictionarySerializer

from reqman.apps.reqtool.models.main_req import Main
from reqman.apps.reqtool.rest_api.serializers.main_req import MainSerializer

from reqman.apps.permissions import IsOwnerOrReadOnly
from reqman.apps.reqtool.rest_api.services import fuseki, inference
from reqman.apps.reqtool.rest_api.services.parse_ontologies import Dictionary_Data_list

from rest_framework.response import Response
from rest_framework.views import APIView

from reqman.apps.reqtool.models.main_req import SHALL_CHOICES, MAIN_SYNTAX

from reqman.apps.reqtool.models.requirements import BoilerplateData, BoilerplateGroupClassesInstances, BoilerplateGroupDictionary
from rest_framework.exceptions import APIException



class MainListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = MainSerializer
    #
    queryset = Main.objects.all()

    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        #print(group_pk)
        queryset_main = Main.objects.filter(boilerplate_of_main = group_pk)
        return queryset_main

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class MainDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]

    serializer_class = MainSerializer
    
    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        #print(group_pk)
        queryset_main= Main.objects.filter(boilerplate_of_main = group_pk)
        return queryset_main

    #Custom actions when PUT
    def perform_update(self, serializer):
        instance = serializer.validated_data
        #add data in thn Model BoilerplateData
        es_instance = instance['sys_fun_inter'].rsplit(':', 1)
        ev_instance =instance['verb'].rsplit(' : ', 1)
        eo_instance = instance['flow_function_interface_item_system_state_stateset'].rsplit(':', 1)
        mn = (es_instance[len(es_instance)-1] + ' ' + instance['shall'] + ' ' + ev_instance[len(ev_instance)-1] + ' ' + eo_instance[len(eo_instance)-1])
        boildata = BoilerplateData.objects.filter(owner_data=instance['boilerplate_of_main'].owner, boilerplate_data_id = instance['boilerplate_of_main'])
        boildata.update(main_data = mn )
        #get instances of the ontology that this boilerplate belongs
        data = BoilerplateGroupClassesInstances.objects.filter(classes_instances_owner= instance['boilerplate_of_main'].owner, classes_instances_group_of_boilerplate = boildata.values('group_of_boilerplate_data')[0]['group_of_boilerplate_data'])
        sub_data = data.values('system_choices')[0]['system_choices'] + data.values('function_choices')[0]['function_choices'] + data.values('inteface_choices')[0]['inteface_choices']
        if( instance['sys_fun_inter'] not in sub_data):
            raise APIException("MAIN : Instance of Subject does not exist")
        stateitem_data = data.values('state_choices')[0]['state_choices'] + data.values('item_choices')[0]['item_choices']
        if( instance['state_item_before_verb'] not in stateitem_data):
            raise APIException("MAIN : Instance of State or Item before Verb does not exist")
        statevalue_data = data.values('state_value_choices')[0]['state_value_choices']
        if( instance['statevalue_before_verb'] not in statevalue_data):
            raise APIException("MAIN : Instance of StateValue before Verb does not exist")
        verb_data = data.values('main_verb_choices')[0]['main_verb_choices']
        if( instance['verb'] not in verb_data):
            raise APIException("MAIN : Instance of Verb does not exist")
        quantifier_data = data.values('quantifier_choices')[0]['quantifier_choices']
        if( instance['quantifier'] not in quantifier_data):
            raise APIException("MAIN : Instance of Quantifier does not exist")
        fifuintsysstatset_data =  data.values('system_choices')[0]['system_choices'] + data.values('function_choices')[0]['function_choices'] + data.values('flow_choices')[0]['flow_choices']  + data.values('inteface_choices')[0]['inteface_choices'] + data.values('item_choices')[0]['item_choices'] + data.values('state_choices')[0]['state_choices'] + data.values('state_set_choices')[0]['state_set_choices']
        if( instance['flow_function_interface_item_system_state_stateset'] not in fifuintsysstatset_data):
            raise APIException("MAIN : Instance of Object does not exist")
        stsysconset_data = data.values('system_choices')[0]['system_choices'] + data.values('state_set_choices')[0]['state_set_choices'] + data.values('connection_choices')[0]['connection_choices'] + data.values('state_value_choices')[0]['state_value_choices']
        if( instance['statevalue_system_connection_stateset'] not in stsysconset_data):
            raise APIException("MAIN : Instance of after Object does not exist")
        numunit_data = data.values('number_unit_choices')[0]['number_unit_choices']
        if( instance['mumerical_units'] not in numunit_data):
            raise APIException("MAIN : Instance of Number Unit does not exist")
        curmain_choices = inference.getclassofmain(instance)
        if curmain_choices[0] == 'ERROR':
            raise APIException("Main choices not in [M1-M16]")
        else:
            instance = serializer.save(main_choices = curmain_choices[0])
        #instance = serializer.save()



    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #instance = serializer.validated_data
        instance.delete()

class MainChoicesViewSet(ListAPIView):
    """
    API view to retrieve, update or delete
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = BoilerplateGroupClassesInstancesSerializer

    def get_queryset(self):
        group_pk = self.kwargs['groupboil']
        queryset_cl= BoilerplateGroupClassesInstances.objects.filter(classes_instances_group_of_boilerplate = group_pk)
        return queryset_cl

        
class ShallChoicesViewSet(APIView):

    def get(self, request):
        response = Response(SHALL_CHOICES)
        return response


#class DictionaryViewSet(APIView):

    #def get(self, request):
    #    response = Response(Dictionary_Data_list)
    #    return response

class DictionaryViewSet(ListAPIView):
    """
    API view to retrieve, update or delete
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = BoilerplateGroupDictionarySerializer

    def get_queryset(self):
        group_pk = self.kwargs['groupboil']
        queryset_cl= BoilerplateGroupDictionary.objects.filter(classes_instances_group_of_boilerplate_dic = group_pk)
        return queryset_cl

class MainSyntaxViewSet(APIView):

    def get(self, request):
        response = Response(MAIN_SYNTAX)
        return response